import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { Completer } from './completer.abstract';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Waypoint } from '../../../types/waypoint.type';
import { CarpoolPathCreator } from '@modules/ad/core/domain/carpool-path-creator.service';
import {
  Point,
  PointProps,
} from '@modules/ad/core/domain/value-objects/point.value-object';

/**
 * Complete candidates with crew carpool path
 */
export class PassengerOrientedCarpoolPathCompleter extends Completer {
  complete = async (
    candidates: CandidateEntity[],
  ): Promise<CandidateEntity[]> => {
    candidates.forEach((candidate: CandidateEntity) => {
      const carpoolPathCreator = new CarpoolPathCreator(
        candidate.getProps().role == Role.DRIVER
          ? candidate.getProps().driverWaypoints.map(
              (waypoint: PointProps) =>
                new Point({
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            )
          : this.query.waypoints.map(
              (waypoint: Waypoint) =>
                new Point({
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            ),
        candidate.getProps().role == Role.PASSENGER
          ? candidate.getProps().driverWaypoints.map(
              (waypoint: PointProps) =>
                new Point({
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            )
          : this.query.waypoints.map(
              (waypoint: Waypoint) =>
                new Point({
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            ),
      );
      candidate.setCarpoolPath(carpoolPathCreator.carpoolPath());
      // console.log(JSON.stringify(candidate, null, 2));
    });
    return candidates;
  };
}
